# Leetcode Populating-Next-Right-Pointers-In-Each-Node

#### 2022-07-16 09:43

> ##### Algorithms:
>
> #algorithm #BFS #optimization #DFS
>
> ##### Data structures:
>
> #DS #binary_tree
>
> ##### Difficulty:
>
> #coding_problems #difficulty_medium
>
> ##### Additional tags:
>
> #leetcode
>
> ##### Revisions:
>
> N/A

##### Related topics:

##### Links:

- [Link to problem](https://leetcode.com/problems/populating-next-right-pointers-in-each-node/)

  ***

### Problem

You are given a **perfect binary tree** where all leaves are on the same level, and every parent has two children. The binary tree has the following definition:

struct Node {
int val;
Node *left;
Node *right;
Node \*next;
}

Populate each next pointer to point to its next right node. If there is no next right node, the next pointer should be set to `NULL`.

Initially, all next pointers are set to `NULL`.

- #### Examples

  ```
  **Example 1:**

  ![](https://assets.leetcode.com/uploads/2019/02/14/116_sample.png)

  **Input:** root = [1,2,3,4,5,6,7]
  **Output:** [1,#,2,3,#,4,5,6,7,#]
  **Explanation:** Given the above perfect binary tree (Figure A), your function should populate each next pointer to point to its next right node, just like in Figure B. The serialized output is in level order as connected by the next pointers, with '#' signifying the end of each level.

  **Example 2:**

  **Input:** root = []
  **Output:** []
  ```

#### Constraints

**Constraints:**

- The number of nodes in the tree is in the range `[0, 212 - 1]`.
- `-1000 <= Node.val <= 1000`

### Thoughts

> [!summary]
> This is a #BFS problem, and cam be optimized
> It relies on the fact that it is perfect binary tree.
> The core part for optimization is utilizing ptr->next

Using ptr->next in DFS and iteration enable us to traverse
between branches like in BFS

![[DFS with links.png]]

##### BFS way of doing this, O(n)time, O(n + 1)space

simple BFS, for each level, connect the node to the next.

Remember to get the for loop right.

##### BFS-like iteration, two pointers, O(n)time, O(n + 1)space

We utilize the ->next property to link
ptr->left->right->next to ptr->right->left

![[Leetcode Populating next right ptr-2.png]]

We can't use ptr->left->right->next = ptr->right->left,
since this is only one level, and doesn't work on two levels:

![[Leetcode Populating next right ptr-1.png]]

##### Also can be done with DFS O(n), O(n)

We recursively fill next pointers the level below current.

Base case:

- Root->left is empty, which means this is the leaf level, quit.

  Pseudo-code:

  1.  Check for base case
  1.  link root->left and root->right
  1.  link root->right and root->next->left (if root->next is valid.)

### Solution

DFS:

```cpp
/*
// Definition for a Node.
class Node {
public:
  int val;
  Node* left;
  Node* right;
  Node* next;

  Node() : val(0), left(NULL), right(NULL), next(NULL) {}

  Node(int _val) : val(_val), left(NULL), right(NULL), next(NULL) {}

  Node(int _val, Node* _left, Node* _right, Node* _next)
      : val(_val), left(_left), right(_right), next(_next) {}
};
*/

class Solution {
void conn(Node *root) {
  // preorder DFS, here we go!
  // It assumes at least one level, which doesn't include []
  if (root->left) {
    root->left->next = root->right;
    if (root->next) {
      root->right->next = root->next->left;
    }
    conn(root->left);
    conn(root->right);
  }
}

public:
Node *connect(Node *root) {
  if (root) {
    conn(root);
  }
  return root;
}
};
```

Iterative, O(n)time O(1)space

```cpp
class Solution {
public:
Node *connect(Node *root) {
  // iteration
  Node *ptr = root;
  Node *nextPtr = nullptr;
  while (ptr && ptr->left) {
    ptr->left->next = ptr->right;
    if (!nextPtr) {
      nextPtr = ptr->left;
    }
    if (!ptr->next) {
      ptr = nextPtr;
      nextPtr = nullptr;
    } else {
      ptr->right->next = ptr->next->left;
      ptr = ptr->next;
    }
  }
  return root;
}
};

```

BFS unoptimized:

```cpp
class Solution {
public:
Node *connect(Node *root) {
  // BFS.
  // Can we do it with DFS?
  queue<Node *> todo;

  if (root && root->left && root->right) {
    todo.push(root->left);
    todo.push(root->right);
  }

  Node *ptr, *nextPtr;

  while (!todo.empty()) {
    nextPtr = todo.front();
    todo.pop();
    for (size_t i = 0, size = todo.size(); i < size; i++) {
      ptr = nextPtr;
      nextPtr = todo.front();
      todo.pop();

      ptr->next = nextPtr;
      if (ptr->left) {
        todo.push(ptr->left);
        todo.push(ptr->right);
      }
    }
    if (nextPtr->left) {
      todo.push(nextPtr->left);
      todo.push(nextPtr->right);
    }
  }

  return root;
}
};
```
